/*
 * Name:     caget.c
 *
 * Purpose:  Program for getting the value of an EPICS process variable.
 *           It is implemented by way of the EZCA toolkit.
 *
 * Author:   William Lavender
 *
 *------------------------------------------------------------------------
 *
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "tsDefs.h"
#include "cadef.h"
#include "ezca.h"

int
main( int argc, char *argv[] )
{
	char *pvname;
	chid *channel_id;
	chtype field_type;
	char ezca_type;
	unsigned element_count, string_size;
	int i, ezca_status;
	void *buffer;
	char *char_ptr, *string_ptr;
	short *short_ptr;
	long *long_ptr;
	float *float_ptr;
	double *double_ptr;

	if ( argc != 2 ) {
		fprintf( stderr, "Usage: caget pvname\n" );
		exit(1);
	}

	pvname = argv[1];

	/* See if the process variable exists by getting its chid. */

	ezca_status = ezcaPvToChid( pvname, &channel_id );

	if ( ezca_status != EZCA_OK )
		exit(1);

	element_count = ca_element_count( *channel_id );

	field_type = ca_field_type( *channel_id );

	if ( field_type < 0 ) {
		fprintf( stderr,
		"caget: The process variable '%s' does not exist.\n",
			pvname );

		exit(1);
	}

	string_size = dbr_size[DBR_STRING];

#if 0
	fprintf( stderr,
		"element_count = %d, field_type = %d, string_size = %d\n",
			element_count, field_type, string_size );
#endif

	switch( field_type ) {
	case DBF_STRING:
		ezca_type = ezcaString;

		buffer = malloc( element_count * string_size + 1 );
		break;
	case DBF_CHAR:
		ezca_type = ezcaByte;

		buffer = malloc( element_count );
		break;
	case DBF_SHORT:
		ezca_type = ezcaShort;

		buffer = malloc( element_count * sizeof(short) );
		break;
	case DBF_LONG:
	case DBF_ENUM:
		ezca_type = ezcaLong;

		buffer = malloc( element_count * sizeof(long) );
		break;
	case DBF_FLOAT:
		ezca_type = ezcaFloat;

		buffer = malloc( element_count * sizeof(float) );
		break;
	case DBF_DOUBLE:
		ezca_type = ezcaDouble;

		buffer = malloc( element_count * sizeof(double) );
		break;
	default:
		fprintf( stderr, "caget: Unsupported field type %d\n",
					(int) field_type );
		exit(1);
	}

	if ( buffer == NULL ) {
		fprintf( stderr,
	"caget: Unable to allocate memory for %d elements in the PV '%s'\n",
			element_count, pvname );

		exit(1);
	}

	ezca_status = ezcaGet( pvname, ezca_type, element_count, buffer );

	if ( ezca_status != EZCA_OK )
		exit(1);

	printf( "%s:   ", pvname );

	switch( ezca_type ) {
	case ezcaString:
		string_ptr = (char *) buffer;

		for ( i = 0; i < element_count; i++ ) {
			printf( "%s ", string_ptr );
			string_ptr += string_size;
		}
		break;
	case ezcaByte:
		char_ptr = (char *) buffer;

		for ( i = 0; i < element_count; i++ ) {
			printf( "0x%x ", *char_ptr );
			char_ptr++;
		}
		break;
	case ezcaShort:
		short_ptr = (short *) buffer;

		for ( i = 0; i < element_count; i++ ) {
			printf( "%hd ", *short_ptr );
			short_ptr++;
		}
		break;
	case ezcaLong:
		long_ptr = (long *) buffer;

		for ( i = 0; i < element_count; i++ ) {
			printf( "%ld ", *long_ptr );
			long_ptr++;
		}
		break;
	case ezcaFloat:
		float_ptr = (float *) buffer;

		for ( i = 0; i < element_count; i++ ) {
			printf( "%g ", *float_ptr );
			float_ptr++;
		}
		break;
	case ezcaDouble:
		double_ptr = (double *) buffer;

		for ( i = 0; i < element_count; i++ ) {
			printf( "%g ", *double_ptr );
			double_ptr++;
		}
		break;
	default:
		fprintf( stderr,
		"caget: Error -- should not be able to get here.\n" );

		exit(1);
	}

	printf( "\n" );

	exit(0);
}

