/*
 * Name:     caput.c
 *
 * Purpose:  Program for setting the value of an EPICS process variable.
 *           It is implemented by way of the EZCA toolkit.
 *
 * Author:   William Lavender
 *
 *------------------------------------------------------------------------
 *
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "tsDefs.h"
#include "cadef.h"
#include "ezca.h"

int
main( int argc, char *argv[] )
{
	char *pvname;
	chid *channel_id;
	chtype field_type;
	char ezca_type;
	unsigned element_count, new_element_count, string_size;
	int i, num_items, ezca_status;
	void *buffer;
	char *char_ptr, *string_ptr;
	short *short_ptr;
	long *long_ptr;
	float *float_ptr;
	double *double_ptr;

	if ( argc <= 2 ) {
		fprintf( stderr, "Usage: caput pvname value\n" );
		exit(1);
	}

	pvname = argv[1];

	/* See if the process variable exists by getting its chid. */

	ezca_status = ezcaPvToChid( pvname, &channel_id );

	if ( ezca_status != EZCA_OK )
		exit(1);

	element_count = ca_element_count( *channel_id );

	if ( argc > ( element_count + 2 ) ) {
		fprintf( stderr,
"caput: Too many arguments specified for the PV '%s'.  The maximum is %d.\n",
			pvname, (int) element_count );

		exit(1);
	}

	field_type = ca_field_type( *channel_id );

	if ( field_type < 0 ) {
		fprintf( stderr,
		"caput: The process variable '%s' does not exist.\n",
			pvname );

		exit(1);
	}

	string_size = dbr_size[DBR_STRING];

#if 0
	fprintf( stderr,
		"element_count = %d, field_type = %d, string_size = %d\n",
			element_count, field_type, string_size );
#endif

	switch( field_type ) {
	case DBF_STRING:
		ezca_type = ezcaString;

		buffer = malloc( element_count * string_size + 1 );
		break;
	case DBF_CHAR:
		ezca_type = ezcaByte;

		buffer = malloc( element_count );
		break;
	case DBF_SHORT:
		ezca_type = ezcaShort;

		buffer = malloc( element_count * sizeof(short) );
		break;
	case DBF_LONG:
	case DBF_ENUM:
		ezca_type = ezcaLong;

		buffer = malloc( element_count * sizeof(long) );
		break;
	case DBF_FLOAT:
		ezca_type = ezcaFloat;

		buffer = malloc( element_count * sizeof(float) );
		break;
	case DBF_DOUBLE:
		ezca_type = ezcaDouble;

		buffer = malloc( element_count * sizeof(double) );
		break;
	default:
		fprintf( stderr, "caput: Unsupported field type %d\n",
					(int) field_type );
		exit(1);
	}

	if ( buffer == NULL ) {
		fprintf( stderr,
	"caput: Unable to allocate memory for %d elements in the PV '%s'\n",
			element_count, pvname );

		exit(1);
	}

	/* Display the old value. */

	ezca_status = ezcaGet( pvname, ezca_type, element_count, buffer );

	if ( ezca_status != EZCA_OK )
		exit(1);

	printf( "Old: %s:   ", pvname );

	switch( ezca_type ) {
	case ezcaString:
		string_ptr = (char *) buffer;

		for ( i = 0; i < element_count; i++ ) {
			printf( "%s ", string_ptr );
			string_ptr += string_size;
		}
		break;
	case ezcaByte:
		char_ptr = (char *) buffer;

		for ( i = 0; i < element_count; i++ ) {
			printf( "0x%x ", *char_ptr );
			char_ptr++;
		}
		break;
	case ezcaShort:
		short_ptr = (short *) buffer;

		for ( i = 0; i < element_count; i++ ) {
			printf( "%hd ", *short_ptr );
			short_ptr++;
		}
		break;
	case ezcaLong:
		long_ptr = (long *) buffer;

		for ( i = 0; i < element_count; i++ ) {
			printf( "%ld ", *long_ptr );
			long_ptr++;
		}
		break;
	case ezcaFloat:
		float_ptr = (float *) buffer;

		for ( i = 0; i < element_count; i++ ) {
			printf( "%g ", *float_ptr );
			float_ptr++;
		}
		break;
	case ezcaDouble:
		double_ptr = (double *) buffer;

		for ( i = 0; i < element_count; i++ ) {
			printf( "%g ", *double_ptr );
			double_ptr++;
		}
		break;
	default:
		fprintf( stderr,
		"caput: Error -- should not be able to get here.\n" );

		exit(1);
	}

	printf( "\n" );

	/* Copy the new value to the outgoing buffer. */

	switch( ezca_type ) {
	case ezcaString:
		string_ptr = (char *) buffer;

		for ( i = 2; i < argc; i++ ) {
			strcpy( string_ptr, "" );
			strncat( string_ptr, argv[i], string_size );
			string_ptr += string_size;
		}
		break;
	case ezcaByte:
		char_ptr = (char *) buffer;

		for ( i = 2; i < argc; i++ ) {

			num_items = sscanf( argv[i], "%x", char_ptr );

			if ( num_items != 1 ) {
				fprintf( stderr,
				"caput: argument '%s' is not in hexadecimal.\n",
					argv[i] );

				exit(1);
			}
		}
		break;
	case ezcaShort:
		short_ptr = (short *) buffer;

		for ( i = 2; i < argc; i++ ) {

			num_items = sscanf( argv[i], "%hd", short_ptr );

			if ( num_items != 1 ) {
				fprintf( stderr,
			"caput: argument '%s' is not a short integer.\n",
					argv[i] );

				exit(1);
			}
		}
		break;
	case ezcaLong:
		long_ptr = (long *) buffer;

		for ( i = 2; i < argc; i++ ) {

			num_items = sscanf( argv[i], "%ld", long_ptr );

			if ( num_items != 1 ) {
				fprintf( stderr,
			"caput: argument '%s' is not a long integer.\n",
					argv[i] );

				exit(1);
			}
		}
		break;
	case ezcaFloat:
		float_ptr = (float *) buffer;

		for ( i = 2; i < argc; i++ ) {

			num_items = sscanf( argv[i], "%g", float_ptr );

			if ( num_items != 1 ) {
				fprintf( stderr,
			"caput: argument '%s' is not a float integer.\n",
					argv[i] );

				exit(1);
			}
		}
		break;
	case ezcaDouble:
		double_ptr = (double *) buffer;

		for ( i = 2; i < argc; i++ ) {

			num_items = sscanf( argv[i], "%g", double_ptr );

			if ( num_items != 1 ) {
				fprintf( stderr,
			"caput: argument '%s' is not a double integer.\n",
					argv[i] );

				exit(1);
			}
		}
		break;
	default:
		fprintf( stderr,
		"caput: Error -- should not be able to get here either.\n" );

		exit(1);
	}

	/* Send the new value and do not wait for a callback. */

	new_element_count = argc - 2;

	ezca_status = ezcaPutOldCa( pvname,
				ezca_type, new_element_count, buffer );

	if ( ezca_status != EZCA_OK )
		exit(1);

	/* Make sure the commands have left the local machine. */

	ca_flush_io();

	/* Display the new value. */

	ezca_status = ezcaGet( pvname, ezca_type, element_count, buffer );

	if ( ezca_status != EZCA_OK )
		exit(1);

	printf( "New: %s:   ", pvname );

	switch( ezca_type ) {
	case ezcaString:
		string_ptr = (char *) buffer;

		for ( i = 0; i < element_count; i++ ) {
			printf( "%s ", string_ptr );
			string_ptr += string_size;
		}
		break;
	case ezcaByte:
		char_ptr = (char *) buffer;

		for ( i = 0; i < element_count; i++ ) {
			printf( "0x%x ", *char_ptr );
			char_ptr++;
		}
		break;
	case ezcaShort:
		short_ptr = (short *) buffer;

		for ( i = 0; i < element_count; i++ ) {
			printf( "%hd ", *short_ptr );
			short_ptr++;
		}
		break;
	case ezcaLong:
		long_ptr = (long *) buffer;

		for ( i = 0; i < element_count; i++ ) {
			printf( "%ld ", *long_ptr );
			long_ptr++;
		}
		break;
	case ezcaFloat:
		float_ptr = (float *) buffer;

		for ( i = 0; i < element_count; i++ ) {
			printf( "%g ", *float_ptr );
			float_ptr++;
		}
		break;
	case ezcaDouble:
		double_ptr = (double *) buffer;

		for ( i = 0; i < element_count; i++ ) {
			printf( "%g ", *double_ptr );
			double_ptr++;
		}
		break;
	default:
		fprintf( stderr,
		"caput: Error -- should not be able to get here at all.\n" );

		exit(1);
	}

	printf( "\n" );

	exit(0);
}

